#!/usr/bin/env python
# -*- Encoding: latin-1 -*-
"""
Tests for the fraction class
 (the mk version, not the mh's one)
written 2004 by Marek Kubica
Licensed under GPL
"""

import unittest
import fractions as fract
__version__ = fract.__version__

class TestRepresentation(unittest.TestCase):
    """Test for checking if the fractions are properly inputed and outputted"""
    
    def testInputConstructor(self):
        """Test that the number constructor works"""
        whole, upper, lower = 1, 1, 2
        f = fract.Fraction(whole=whole, upper=upper, lower=lower)
        if fract.extractWhole(f.getMixedNumber()) != whole:
            self.fail('setting whole failed')
        if fract.extractUpper(f.getMixedNumber()) != upper:
            self.fail('setting upper failed')
        if fract.extractLower(f.getMixedNumber()) != lower:
            self.fail('setting lower failed')
    
    def testInputConstructorString(self):
        """Test that the string constructor works"""
        # alternate constructor - uses directly strings
        w, u, l = 1, 1, 2
        #raise NotImplementedError
        string_input = str(w) + ' ' + str(u) + '/' + str(l)
        f = fract.Fraction(string_input)
        if fract.extractWhole(f.getMixedNumber()) != w:
            self.fail('setting whole failed')
        elif fract.extractUpper(f.getMixedNumber()) != u:
            self.fail('setting upper failed')
        elif fract.extractLower(f.getMixedNumber()) != l:
            self.fail('setting lower failed')
    
    def testInputConstructorEmpty(self):
        """Test that the default values are set on construction"""
	# these are the default values (from the constructor of Fraction)
	# test also the use of reduce
        default = {'whole' : 0, 'upper' : 0, 'lower' : 1}
        f = fract.Fraction()
        if fract.extractWhole(f.getMixedNumber()) != default['whole']:
            self.fail('setting whole failed')
        if fract.extractUpper(f.getMixedNumber()) != default['upper']:
            self.fail('setting upper failed')
        if fract.extractLower(f.getMixedNumber()) != default['lower']:
            self.fail('setting lower failed')
    
    def testInputString(self):
        """Test the internal input of string fractions"""
        w, u, l = 1, 1, 2
        string_input = str(w) + ' ' + str(u) + '/' + str(l)
        f = fract.Fraction()
	# this is deprecated to use external
        f._Fraction__input(string_input)
        
        l_new = fract.extractLower(f.getMixedNumber())
	w_new = fract.extractWhole(f.getMixedNumber())
	u_new = fract.extractUpper(f.getMixedNumber())
        self.assertEqual(w, w_new, 'Whole does not compare equal')
        self.assertEqual(u, u_new, 'Upper does not compare equal')
        self.assertEqual(l, l_new, 'Lower does not compare equal')
    
    def testInputStringBad(self):
        """Test that input of bad string fractions gives errors"""
        w, u, l = 'a', 1, 2
        
        string_input = str(w) + ' ' + str(u) + '/' + str(l)
        f = fract.Fraction()
        
        # run f._Fraction__input(string_input) and look for ValueError
	# running _Fraction__input is deprecated from external
        self.assertRaises(ValueError, f._Fraction__input, string_input)
    
    def testInputStringLowerZero(self):
        """Test that it is possible to internally set negative fractions using strings"""
        f = fract.Fraction()
        w, u, l = -1, 1, 2
        string_input = str(w) + ' ' + str(u) + '/' + str(l)
	# this is deprecated to use externally
        f._Fraction__input(string_input)
        string_output = f.getMixedNumber()
        
        self.assertEqual(string_input, string_output, 'String streams not equal')
        #self.fail('test not ready')

    def testInputStringWholeNumber(self):
	"""Test setting a simple whole number using strings"""
	val_norm = '10'
	f = fract.Fraction(val_norm)
	val_prog = f.getMixedNumber()
	self.assertEqual(val_prog, val_norm, 'The value returned is bad')
	
    def testInputWholeNumber(self):
	"""Test setting a simple whole number"""
	val_norm = 10
	f = fract.Fraction(val_norm)
	val_prog = int(f.getMixedNumber())
	self.assertEqual(val_prog, val_norm, 'The value returned is bad')
    
    def testOutputString(self):
        """Test if the output string is correct"""
        # should this mean inputstring == outputstring or can the factors be reduced?

	input_string = '3 3/4'
	f = fract.Fraction(input_string)
	output_string = f.getMixedNumber()
	self.assertEqual(input_string, output_string, 'The returned string representation is bad')
    
    def testOutputPrettyprint(self):
	"""Test if the fraction prettyprinting is working"""
	inpt = '3 11/20'
	f = fract.Fraction(inpt)
	# use the internal __getPrettyprint function
	otpt = f._Fraction__getPrettyprint()
	# the prettyprinted string should be:
	pprint = '  11\n3 --\n  20'
	
	# look if the strings are the same
	self.assertEqual(pprint, otpt, 'Prettyprint is bad')
    
    def testSetWhole(self):
        """Test that the internal whole sets right values"""
        w, u, l = 1, 1, 2
        f = fract.Fraction(whole=w, upper=u, lower=l)
        w_new = fract.extractWhole(f.getMixedNumber())
        self.assertEqual(w, w_new)
    
    def testSetWholeBad(self):
	"""Test setting a bad 'whole' value"""
	w, u, l = 'trash', 1, 2
	#f = fract.Fraction(whole=w, upper=u, lower=l)
	self.assertRaises(ValueError, fract.Fraction, whole=w, upper=u, lower=l)
    
    def testSetUpper(self):
        """Test that the internale upper sets right values"""
        w, u, l = 1, 1, 2
        f = fract.Fraction(whole=w, upper=u, lower=l)
	u_new = fract.extractUpper(f.getMixedNumber())
        self.assertEqual(u, u_new)
    
    def testSetUpperBad(self):
        """Test that it is not possible to set bad values for upper"""
        w, u, l = 1, 'a', 2
        # ^^ upper is a string, do you see it?
        
        # try if we get a ValueError when we try to initialize with bad values (indriectly calling setUpper())
        self.assertRaises(ValueError, fract.Fraction, whole=w, upper=u, lower=l)
    
    def testSetLower(self):
        """Test that the internal lower sets right values"""
        w, u, l = 1, 1, 2
        f = fract.Fraction(whole=w, upper=u, lower=l)
        l_new = fract.extractLower(f.getMixedNumber())
        self.assertEqual(l, l_new)

    
    def testSetLowerBad(self):
        """Test that it is not possible to set bad values for lower"""
        w, u, l = 1, 1, 'a'
        # ^^ lower is a string, do you see it?
        
        # try if we get a ValueError when we try to initialize with bad values (indriectly calling setLower())
        self.assertRaises(ValueError, fract.Fraction, whole=w, upper=u, lower=l)
    
    def testSetLowerBadZero(self):
        """Test that is is not possible to set zero values for lower"""
        w, u, l = 1, 1, 0
        # try to create a fraction with a lower == 0 value
        self.assertRaises(ValueError, fract.Fraction, whole=w, upper=u, lower=l)
        # ^^ Indriectly calling __getLower() by __init__()
        
    def testNewRepresentation(self):
	"""Test the representation of the fraction"""
	fr = '3 3/4'
	f = fract.Fraction(fr)
	val_gv = f.getMixedNumber()
	val_repr = repr(f)
	self.assertEqual(val_gv, val_repr)

    def testZeroWholeGetMixedNumber(self):
	"""Test getting a whole value by getting the mixed number representation"""
	w, u, l = 0, 4, 5
	f = fract.Fraction(whole=w, upper=u, lower=l)
	w_new = fract.extractWhole(f.getMixedNumber())
	self.assertEqual(w, w_new, 'Whole is non-zero!')

    def testZeroWholeStringExtractWhole(self):
	"""Test extracting a zero whole value"""
	# this test has to be moved elsewhere
	fract_fract = '4/5'
	w = 0
	f = fract.Fraction(fract_fract)
	w_new = fract.extractWhole(f.getMixedNumber())
	self.assertEqual(w, w_new, 'Whole is non-zero!')
	
    def testGetMixedNumber(self):
	"""Test getting mixed numbers"""
	# this test has to be moved
	fract_fract = '7/2'
	fract_mixed = '3 1/2'
	f = fract.Fraction(fract_fract)
	fract_mixed_new = f.getMixedNumber()
	self.assertEqual(fract_mixed, fract_mixed_new, 'Getting mixed numbers failed')
    
class TestFunction(unittest.TestCase):
    """Tests for functions to run on Fraction instances"""
    def testFloat(self):
        """Test that the fractions are flattened to proper float values"""
        w, u, l = 1, 1, 2
        fract_float = 1.5
        f = fract.Fraction(whole=w, upper=u, lower=l)
	
	try:
	    fract_float_new = float(f)
	except TypeError, e:
	    self.fail('Problems with __float__ hook: ' + str(e))
        self.assertEqual(fract_float, fract_float_new, 'Errors while converting to float')

    def testInt(self):
	"""Test that the fractions are flattened to proper int values"""
	fract_fract = '1 1/4'
	fract_int = 1
	f = fract.Fraction(fract_fract)
	try:
	    fract_int_new = int(f)
	except TypeError, e:
	    self.fail('Problems with __int__ hook: ' + str(e))
	self.assertEqual(fract_int, fract_int_new, 'Errors while converting to int')

    def testStr(self):
	"""Test the string version of the fraction"""
	fract_str = '1 1/4'
	f = fract.Fraction(fract_str)
	try:
	    fract_str_new = str(f)
	except TypeError, e:
	    self.fail('Problems with __repr__ hook: ' + str(e))
	self.assertEqual(fract_str, fract_str_new, 'Errors while converting to string')



    def testInternalMixed(self):
	"""Test if unreal fractions can be converted to mixed numbers"""
	fract_fract = '15/4'
	fract_mixed = '3 3/4'
	f = fract.Fraction(fract_fract)
	f._Fraction__mixed()
	fract_mixed_new = f.getMixedNumber()
	self.assertEqual(fract_mixed, fract_mixed_new, 'Convertion into mixed numbers faulty')

    def testHeavilyMixed(self):
	"""Test if unreal fractions with wholes can be converted to mixed numbers"""
	fract_fract = '3 15/4'
	fract_mixed = '6 3/4'
	f = fract.Fraction(fract_fract)
	f._Fraction__mixed()
	fract_mixed_new = f.getMixedNumber()
	self.assertEqual(fract_mixed, fract_mixed_new, 'Convertion into mixed numbers faulty')
	
    def testSimpleReduce(self):
	"""Test simple reducing"""
	fract_bigger = '2/4'
	fract_reduced = '1/2'
	f = fract.Fraction(fract_bigger)
	f.reduce()
	fract_reduced_new = f.getMixedNumber()
	self.assertEqual(fract_reduced, fract_reduced_new, 'Reducing failed')

    def testInternalUnreal(self):
	"""Test the function of returning unreal fractions"""
	fract_fract = '3 3/4'
	fract_unreal = '15/4'
	f = fract.Fraction(fract_fract)
	fract_unreal_new = f.getUnrealNumber()
	self.assertEqual(fract_unreal, fract_unreal_new, 'Convertion into unreal fractions faulty')


class TestOperation(unittest.TestCase):
    # test adding, dividing and so on (overloaded types)

    def testAddFractionDifferentLower(self):
	"""Test adding a Fraction to a Fraction with different lower values"""
	fract1 = '3/4'
	fract2 = '1/5'
	# the result
	fract3 = '19/20'
	f1 = fract.Fraction(fract1)
	f2 = fract.Fraction(fract2)
	f3 = f1 + f2
	fract3_new = f3.getMixedNumber()
	self.assertEqual(fract3, fract3_new, 'Adding failed')

    def testAddFractionSameLower(self):
	"""Test adding a Fraction to a Fraction with the same lower values"""
	fract1 = '10/20'
	fract2 = '5/20'
	# the result (already reduced)
	fract3 = '3/4'
	f1 = fract.Fraction(fract1)
	f2 = fract.Fraction(fract2)
	f3 = f1 + f2
	fract3_new = f3.getMixedNumber()
	self.assertEqual(fract3, fract3_new, 'Adding failed')

class TestExtracts(unittest.TestCase):
    def testExtractMixedWhole(self):
	"""Test extracting the whole value from a mixed number"""
	realwhole = 3
	u, l = 5, 7
	f = fract.Fraction(realwhole, u, l)
	extract = fract.extractWhole(f.getMixedNumber())
	self.assertEqual(realwhole, extract, 'Whole extraction from mixed numbers faulty')

    def testExtractUnrealWhole(self):
	"""Test extracting the whole value from an unreal fraction"""
	u, l = 5, 7
	f = fract.Fraction(upper=u, lower=l)
	extract = fract.extractWhole(f.getUnrealNumber())
	# the extract should be zero
	self.assertEqual(extract, 0, 'Whole in unreal fraction should ever be zero')
    
    def testExtractMixedUpper(self):
	"""Test extracting the upper value from a mixed number"""
	realupper = 1
	fract_mixed = '3 ' + str(realupper) + '/4'
	
	f = fract.Fraction(fract_mixed)
	extract = fract.extractUpper(f.getMixedNumber())
	self.assertEqual(realupper, extract, 'Upper extraction from mixed numbers faulty')

    def testExtractUnrealUpper(self):
	"""Test extracting the upper value from an unreal fraction"""
	u, l = 7, 5
	f = fract.Fraction(upper=u, lower=l)
	extract = fract.extractUpper(f.getUnrealNumber())
	self.assertEqual(u, extract, 'Upper extraction from an unreal fraction failed')

    def testExtractMixedLower(self):
	"""Test extracting the lower value from a mixed number"""
	reallower = 4
	fract_mixed = '3 1/' + str(reallower)
	
	f = fract.Fraction(fract_mixed)
	extract = fract.extractLower(f.getMixedNumber())
	self.assertEqual(reallower, extract, 'Lower extraction from mixed numbers faulty')
	
    def testExtractUnrealLower(self):
	"""Test extracting the lower value from an unreal fraction"""
	u, l = 7, 5
	f = fract.Fraction(upper=u, lower=l)
	extract = fract.extractLower(f.getUnrealNumber())
	self.assertEqual(l, extract, 'Upper extraction from an unreal fraction failed')

    def testExtractZeroFractionUpper(self):
	"""Test extracting the upper value from a fraction without fraction part"""
	w = 10
	f = fract.Fraction(w)
	# getUnrealnumber makes other extracts...
	extract = fract.extractUpper(f.getMixedNumber())
	self.assertEqual(extract, 0, 'Upper should be zero if no fraction part')

    def testExtractZeroFractionLower(self):
	"""Test extracting the lower value from a fraction without fraction part"""
	w = 10
	f = fract.Fraction(w)
	# getUnrealnumber makes other extracts...
	extract = fract.extractLower(f.getMixedNumber())
	# one - not zero, since lower is *never* zero
	self.assertEqual(extract, 1, 'Lower should be one if no fraction part')

if __name__ == '__main__':
    unittest.main()